""" Auditd reactive charm layer

This file is part of auditd reactive charm layer.
Copyright 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3, as published by the
Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import grp
import os
import pwd
import shutil
import subprocess
import yaml

from charms.reactive import helpers, hook, when, when_not
from charms.reactive import set_state, remove_state
from charmhelpers import fetch
from charmhelpers.core import hookenv, host, templating


SERVICE_NAME = 'auditd'


@when('audit-relation.broken')
def break_audit_relation(relation):
    if relation.name is not None:
        path = '/etc/audit/rules.d/{}.rules'.format(relation.name)
        if os.path.exists(path):
            os.remove(path)
            start_auditd(restart=True)


@hook('config-changed')
def configure_auditd():
    hookenv.status_set('maintenance', 'Configuring')

    config = hookenv.config()
    config.get('log_file')

    # Write primary config
    config_files = ['/etc/audit/auditd.conf']
    templating.render(source='auditd.conf.j2',
                      target='/etc/audit/auditd.conf',
                      owner='root',
                      perms=0o640,
                      context=config)
    # Write audispd syslog config
    config_files.append('/etc/audisp/plugins.d/syslog.conf')
    templating.render(source='audisp.syslog.conf.j2',
                      target='/etc/audisp/plugins.d/syslog.conf',
                      owner='root',
                      perms=0o640,
                      context=config)

    # Setup rules and augenrules
    config_files.append(copy_file('auditd.default', '/etc/default/auditd'))
    config_files.append(copy_file('base.rules',
                                  '/etc/audit/rules.d/base.rules'))

    config_files.append('/etc/audit/rules.d/additional.rules')
    rules = yaml.safe_load(config.get('additional_rules', '[]'))
    templating.render(source='additional.rules.j2',
                      target='/etc/audit/rules.d/additional.rules',
                      owner='root',
                      perms=0o640,
                      context={'rules': rules})

    assert subprocess.call(['/sbin/augenrules']) == 0, "Augenrules Error"

    changed = helpers.any_file_changed(config_files)
    if changed:
        assert subprocess.call(['/sbin/augenrules']) == 0, \
               "Error running augenrules"

    set_state('auditd.configured')
    # restart only on change but make sure the app is started everytime
    start_auditd(restart=changed)


def copy_file(source_file, dest, owner='root', group='root', mode=0o640,
              source_dir=None):
    """ Copy a file from src/dest with the source_dir defaulting to files in
        the charm.
        dest is expected to be an absolute path.
        Returns the dest path if successful.
    """
    if source_dir is None:
        source_dir = os.path.join(hookenv.charm_dir(), 'files')

    shutil.copy(os.path.join(source_dir, source_file), dest)
    os.chown(dest, pwd.getpwnam(owner).pw_uid, grp.getgrnam(group).gr_gid)
    os.chmod(dest, mode)
    return dest


@hook('install')
def install_auditd():
    hookenv.status_set('maintenance', 'Installing')
    fetch.apt_update()
    fetch.apt_install('auditd')
    # Note: The deb will enable and start the service
    set_state('auditd.installed')


@when('audit-relation.available')
def setup_relation_rules(relation):
    """Configures any specific rules provided by the parent charm through the
       audit relation.  The charm providing the relation needs to set name and
       rules in its joined relation.
    """
    path = '/etc/audit/rules.d/{}.rules'.format(relation.name)
    templating.render(source='additional.rules.j2',
                      target=path,
                      owner='root',
                      perms=0o640,
                      context={'rules': relation.rules})

    changed = helpers.any_file_changed([path])
    if changed:
        assert subprocess.call(['/sbin/augenrules']) == 0, \
               "Error running augenrules"
    # restart only on change but make sure the app is started everytime
    start_auditd(restart=changed)


@when('nrpe-external-master.available')
def setup_nrpe_checks(nagios):
    config = hookenv.config()
    # Default time is 240 seconds for warn, 600 seconds for critical plenty of
    # time for an event with the default auditd profile
    nagios.add_check(['/usr/lib/nagios/plugins/check_file_age', '-f',
                      config['log_file']],
                     name="check_auditd_log",
                     description="Verify auditd is running/logging properly",
                     context=config["nagios_context"],
                     unit=hookenv.local_unit())


@when('auditd.configured')
@when_not('auditd.started')
def start_auditd(restart=False):
    hookenv.status_set('maintenance', 'Starting')
    if not host.service_running(SERVICE_NAME):
        assert host.service_start(SERVICE_NAME), \
               'Failed to start {}'.format(SERVICE_NAME)
    elif restart:
        assert host.service_restart(SERVICE_NAME), \
               'Failed to restart {}'.format(SERVICE_NAME)

    # The auditd init script in trusty annoyingly exits 0 even on failure
    if not host.service_running(SERVICE_NAME):
        raise AssertionError('Failed to start {}'.format(SERVICE_NAME))

    set_state('auditd.started')
    hookenv.status_set('active', 'Ready')


@hook('stop')
def stop_auditd():
    if host.service_running(SERVICE_NAME):
        assert host.service_stop(SERVICE_NAME), \
               'Failed to stop {}'.format(SERVICE_NAME)

    remove_state('auditd.started')
    remove_state('auditd.configured')
    remove_state('auditd.installed')
    fetch.apt_purge('auditd')

    if os.path.exists('/etc/audit'):
        shutil.rmtree('/etc/audit')

    if os.path.exists('/etc/audisp'):
        shutil.rmtree('/etc/audisp')

    hookenv.status_set('blocked', 'Stopped')
